"""
A simple test problem for the Resource Constrained Scheduling Solver OptSeq
file name: Example11.py
Copyright Log Opt Co., Ltd.

Consider a small job shop problem consisting of 4 activities and 3 machines.
Each activity consists of three sub-jobs (operations).
Operation must be processed in the order of 1,2 and 3 on thecorresponding machines.
The machines to be processed and the processing time (days) are kept by the dictionary  
JobInfo={   (1,1):(1,7), (1,2):(2,10), (1,3):(3,4),
            (2,1):(3,9), (2,2):(1,5), (2,3):(2,11),
            (3,1):(1,3), (3,2):(3,9), (3,3):(2,12),
            (4,1):(2,6), (4,2):(3,13),(4,3):(1,9)
            }
that maps a tuple of activity ID and operation ID to a tuple of machine IDand processing time.
The objective is to minimize the latest completion time of activities (makespan).
We add the following practical conditions:
1. Each operation requires the worker resource for the first 2 days.
The worker resource is available on weekdays only and its capacity(maximum number of operations to be executed in a day) is 2.
2. Each operation may have a break after 1 day.
3. Operations on machine 1 can be processed in parallel. 
4. Operations on machine 2 can be processed in an express mode whose processing time is 4 days.
The express mode is restricted to be at most once in a whole. 
5. On machine 1, operation 2 must be executed just after operation 1.
"""
from optseq import *
m1=Model()
#resource declaration  
machine={} #define three machines
for j in range(1,4):
    machine[j]=m1.addResource("machine[{0}]".format(j),capacity={(0,"inf"):1})

#CAP={} #capacity of human resources; two workers are available on weekdays
#for t in range(9):
#    CAP[(t*7,t*7+5)]=2
#manpower=m1.addResource("manpower",capacity=CAP)
# we may write ...
manpower=m1.addResource("manpower")
for t in range(9):
    manpower.addCapacity(t*7,t*7+5,2)

#budget constraint
budget=m1.addResource("budget_constraint",rhs=1)

#activity declaration (3 activities are processed on three machines)
#JobInfo containes the info. of operations (activity,1..3):-> machine ID and proc. time 
JobInfo={ (1,1):(1,7), (1,2):(2,10), (1,3):(3,4),
          (2,1):(3,9), (2,2):(1,5), (2,3):(2,11),
          (3,1):(1,3), (3,2):(3,9), (3,3):(2,12),
          (4,1):(2,6), (4,2):(3,13), (4,3):(1,9)
    }
act={}
express=Mode("Express",duration=4)
express.addResource(machine[2],{(0,"inf"):1},"max")
express.addResource(manpower,{(0,2):1})
express.addBreak(1,1)
#express.addParallel(1,1,2)
    
mode={}
for (i,j) in JobInfo: #for each job and machine
    act[i,j]=m1.addActivity("Act[{0}][{1}]".format(i,j))
    mode[i,j]=Mode("Mode[{0}][{1}]".format(i,j),duration=JobInfo[i,j][1])
    mode[i,j].addResource(machine[JobInfo[i,j][0]],{(0,"inf"):1},"max")
    mode[i,j].addResource(manpower,{(0,2):1})
    mode[i,j].addBreak(1,1)
    if JobInfo[i,j][0]==1:
        mode[i,j].addParallel(1,1,2)

    
    if JobInfo[i,j][0]==2:
        #activities processed on machine 2 have two modes, Express and Normal.
        act[i,j].addModes(mode[i,j],express)
        #Express mode needs one budget
        budget.addTerms(1,act[i,j],express)
    else:
        act[i,j].addModes(mode[i,j]) #single mode activity
    #print act[i,j]
#temporal (precedense) constraints 
for i in range(1,5):
    for j in range(1,3):
        m1.addTemporal(act[i,j],act[i,j+1])   

#Define that Act[2][2] must be processed just after Act[1][1] on machine 1
#introduce dummy with duration 0 and can break at time 0
#it requires machine 1 during the break,
#  completion of Act[1][1]=start of dummy and completaion of dummy=start of Act[2][2]  
d_act=m1.addActivity("dummy_activity")
d_mode=Mode("dummy_mode")
d_mode.addBreak(0,0)
d_mode.addResource(machine[1],{(0,0):1},"break")
d_act.addModes(d_mode)
m1.addTemporal(act[1,1],d_act,tempType="CS")
m1.addTemporal(d_act,act[1,1],tempType="SC")
m1.addTemporal(d_act,act[2,2],tempType="CS")
m1.addTemporal(act[2,2],d_act,tempType="SC")

print (m1)
m1.Params.TimeLimit=1
m1.Params.OutputFlag=True
m1.Params.Makespan=True
m1.optimize()
m1.write("chart11.txt")
